@@ -1,9 +1,10 @@
-# diffobj - Diffs for R Objects
 # Copyright (C) 2016  Brodie Gaslam
 #
+# This file is part of "diffobj - Diffs for R Objects"
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
+# the Free Software Foundation, either version 2 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -11,7 +12,7 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# Go to <https://www.r-project.org/Licenses/GPL-3> for a copy of the license.
+# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.
 
 is.less_flags <-
   function(x) is.chr.1L(x) && isTRUE(grepl("^[[:alpha:]]*$", x))
@@ -123,6 +124,14 @@ is.one.file.name <- function(x) {
     sprintf("(\"%s\") is not a file", x)
   } else TRUE
 }
+is.non.obj.style <- function(x)
+  string_in(x, "auto") || (is.list(x) && !is.object(x))
+
+# Things that could possibly be output by substitute
+
+is.possibly.substituted <- function(x)
+  (is.atomic(x) && length(x) == 1L) || is.null(x) || is.name(x) || is.call(x)
+
 # Checks common arguments across functions
 
 check_args <- function(
@@ -130,11 +139,32 @@ check_args <- function(
   color.mode, pager, ignore.white.space, max.diffs, align, disp.width,
   hunk.limit, convert.hz.white.space, tab.stops, style, palette.of.styles,
   frame, tar.banner, cur.banner, guides, rds, trim, word.diff, unwrap.atomic,
-  extra, interactive, term.colors
+  extra, interactive, term.colors, call.match
 ) {
   err <- make_err_fun(call)
   warn <- make_warn_fun(call)
 
+  # Check for conflicting arguments
+
+  formals <- tail(names(call.match), -1L)
+  style.overrideable <- c("format", "brightness", "color.mode")
+  if(
+    "style" %in% formals && !is.non.obj.style(style) &&
+    any(s.ov <- style.overrideable %in% formals)
+  )
+    warn(
+      "Provided `style` argument will override the provided ",
+      if(sum(s.ov) < 2L) {
+        sprintf("`%s` argument", style.overrideable[s.ov])
+      } else {
+        paste0(
+          paste0(
+            sprintf("`%s`", head(style.overrideable[s.ov], -1L)),
+            collapse=", "
+          ),
+          " and `", tail(style.overrideable[s.ov], 1L), "` arguments."
+    ) } )
+
   # Check extra
 
   if(!is.list(extra)) err("Argument `extra` must be a list.")
@@ -152,8 +182,24 @@ check_args <- function(
     err(sprintf(msg.base, "context"))
 
   if(!is(context, "AutoContext")) {
-    context <- if(identical(context, "auto")) auto_context() else
-      auto_context(as.integer(context), as.integer(context))
+    context <- if(identical(context, "auto")) auto_context() else {
+      if(is.int.1L(context) && context < 0) {
+        min.cont <- 0
+        max.cont <- -1
+      } else if (is.int.1L(context)) {
+        min.cont <- max.cont <- as.integer(context)
+      } else {
+        err("Argument `context` must be integer(1L) and not NA.")
+      }
+      cont <- try(auto_context(min.cont, max.cont))
+      if(inherits(cont, "try-error"))
+        err(
+          "Unable to instantiate an `AutoContext` object from provided ",
+          "`context` argument.  Value should be integer(1L) and not NA, or ",
+          "an `AutoContext` object as generated by `auto_context()`."
+        )
+      cont
+    }
   }
   # any 'substr' of them otherwise these checks fail
 
@@ -224,13 +270,23 @@ check_args <- function(
       err(sprintf(msg.base, "max.diffs"))
     assign(x, as.integer(int.val))
   }
-  # char or NULL vars
+  # Banners; convolution here is to accomodate `diffObj` and have it be able
+  # to pass captured target/current expressions
 
-  chr1LorNULL.vars <- c("tar.banner", "cur.banner")
-  msg.base <- "Argument `%s` must be character(1L) and not NA, or NULL"
-  for(x in chr1LorNULL.vars) {
+  chr1LorNULLorLanguage.vars <- c("tar.banner", "cur.banner")
+  msg.base <-
+    "Argument `%s` must be atomic and length(1L), NULL, a symbol, or a call"
+  for(x in chr1LorNULLorLanguage.vars ) {
     y <- get(x, inherits=FALSE)
-    if(!is.chr.1L(y) && !is.null(y)) err(sprintf(msg.base, x))
+    if(!is.possibly.substituted(y)) err(sprintf(msg.base, x))
+  }
+  if(!is.chr.1L(tar.banner) && !is.null(tar.banner)) {
+    tar.exp <- tar.banner
+    tar.banner <- NULL
+  }
+  if(!is.chr.1L(cur.banner) && !is.null(cur.banner)) {
+    cur.exp <- cur.banner
+    cur.banner <- NULL
   }
   # Align threshold
 
@@ -254,7 +310,7 @@ check_args <- function(
     !is(style, "Style") && !string_in(style, "auto") &&
     !(is.list(style) && !is.object(style))
   )
-    err("Argument `style` must be \"auto\",  a `Style` object, or a list.")
+    err("Argument `style` must be \"auto\", a `Style` object, or a list.")
 
   # pager
 
@@ -306,6 +362,13 @@ check_args <- function(
       style <- "auto"
     } else style.args <- list()
 
+    # We only want to allow ansi styles if the pager supports them too;
+    # unfortuantely we cannot have different styles depending on whether the
+    # output is paged or not, at least not at this time
+
+    pager.could.be.ansi <- if(is(pager, "Pager"))
+      pager@ansi else pager_is_less()
+
     if(!is.chr.1L(format))
       err("Argument `format` must be character(1L) and not NA")
     valid.formats <- c("auto", dimnames(palette.of.styles@data)$format)
@@ -319,7 +382,7 @@ check_args <- function(
         )
       # No recognized color alternatives, try to use HTML if we can
 
-      format <- if(!term.colors %in% c(8, 256)) {
+      format <- if(!term.colors %in% c(8, 256) || !pager.could.be.ansi) {
         if(
           interactive && (identical(pager, "on") || is(pager, "PagerBrowser"))
         ) "html" else "raw"
