@@ -1,9 +1,10 @@
-# diffobj - Diffs for R Objects
 # Copyright (C) 2016  Brodie Gaslam
 #
+# This file is part of "diffobj - Diffs for R Objects"
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
+# the Free Software Foundation, either version 2 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -11,7 +12,7 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# Go to <https://www.r-project.org/Licenses/GPL-3> for a copy of the license.
+# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.
 
 # Split by guides; used by nested structures to retrieve contents within
 # guides.  Each element has an attribute indicating the indices from the
@@ -226,6 +227,41 @@ detect_array_guides <- function(txt, dim.n) {
       dim.guide.fin else integer(0L)
   } else integer(0L)
 }
+# Utility fun to determin whether an object would be shown with the default show
+# method
+
+is_default_show_obj <- function(obj) {
+  stopifnot(isS4(obj))
+  s.m <- selectMethod("show", class(obj))
+  identical(
+    class(s.m),
+    structure("derivedDefaultMethod", package = "methods")
+  )
+}
+# Basic S4 guide detection, does not handle nesting or anything fancy like that
+# and could easily be fooled
+
+detect_s4_guides <- function(txt, obj) {
+  stopifnot(isS4(obj))
+
+  # Only try to do this if relying on default S4 show method
+
+  if(is_default_show_obj(obj)) {
+    # this could be an issue if they start using curly quotes or whatever...
+    guides <- c(
+      sprintf("An object of class \"%s\"", class(obj)),
+      sprintf("Slot \"%s\":", slotNames(obj))
+    )
+    guides.loc <- which(txt %in% guides)
+    guides.txt <- txt[guides.loc]
+
+    if(!identical(guides, guides.txt)) {
+      integer()
+    } else {
+      guides.loc
+    }
+  } else integer()
+}
 #' Generic Methods to Implement Flexible Guide Line Computations
 #'
 #' Guides are context lines that would normally be omitted from the
@@ -259,19 +295,31 @@ detect_array_guides <- function(txt, dim.n) {
 #' character representation thereof.
 #'
 #' The default method for \code{guidesPrint} has special handling for 2D
-#' objects (e.g. data frames, matrices), arrays, time series, tables, and lists.
+#' objects (e.g. data frames, matrices), arrays, time series, tables, lists, and
+#' S4 objects that use the default \code{show} method.  Guide finding is on a
+#' best efforts basis and may fail if your objects contain \dQuote{pathological}
+#' display representations.  Since the diff will still work with failed
+#' \code{guides} finding we consider this an acceptable compromise.  Guide
+#' finding is more likely to fail with nested recursive structures.
+#'
+#' \code{guidesStr} highlights top level objects.  The default methods for the
+#' other \code{guide*} generics do not do anything and exist only as a mechanism
+#' for providing custom guide line methods.
+#'
 #' If you dislike the default handling you can also define your own methods for
 #' matrices, arrays, etc., or alternatively you can pass a guide finding
 #' function directly via the \code{guides} parameter to the \code{diff*}
-#' methods.  The default method for \code{guidesStr} highlights top level
-#' objects.  The default methods for the other \code{guide*} methods
-#' don't do anything and exist only as a mechanism for providing custom guide
-#' line methods.
+#' methods.
 #'
 #' If you have classed objects with special patterns you can define your own
 #' methods for them (see examples), though if your objects are S3 you will need
 #' to use \code{\link{setOldClass}} as the \code{guides*} generics are S4.
 #'
+#' @note The mechanism for identifying guides will almost certainly change in
+#'   the future to allow for better handling of nested guides, so if you do
+#'   implement custom guideline methods do so with the understanding that they
+#'   will likely be deprecated in one of the future releases.
+#'
 #' @aliases guidesPrint, guidesStr, guidesChr, guidesDeparse
 #' @rdname guides
 #' @name guides
@@ -320,6 +368,8 @@ setMethod(
       detect_array_guides(obj.as.chr, dimnames(obj))
     } else if (is.list(obj)) {
       detect_list_guides(obj.as.chr)
+    } else if (isS4(obj)) {
+      detect_s4_guides(obj.as.chr, obj)
     } else integer(0L)
   }
 )
@@ -383,11 +433,17 @@ apply_guides <- function(obj, obj.as.chr, guide_fun) {
   msg.extra <- paste0(
     "If you did not define custom `guides*` methods contact maintainer."
   )
-  if(inherits(guide, "try-error"))
-    stop(
+  msg.extra <- paste0(
+    "If you did not specify a `guides` function or define custom `guides*` ",
+    "methods contact maintainer (see `?guides`).  Proceeding without guides."
+  )
+  if(inherits(guide, "try-error")) {
+    warning(
       "`guides*` method produced an error when attempting to compute guide ",
-      "lines; ", msg.extra
+      "lines ; ", msg.extra
     )
+    guide <- integer()
+  }
   if(
     !is.integer(guide) || anyNA(guide) || anyDuplicated(guide) ||
     !all(guide %in% seq_along(obj.as.chr))
